# Generated by roxygen2: do not edit by hand

S3method(makeContent,make_boxplot)
S3method(makeContent,makeci)
S3method(makeContext,make_boxplot)
S3method(makeContext,makeci)
S3method(plot,forestplot)
S3method(print,forestplot)
export(add_border)
export(add_grob)
export(add_text)
export(edit_plot)
export(forest)
export(forest_theme)
export(get_wh)
export(insert_text)
export(make_boxplot)
export(make_summary)
export(makeci)
import(grid)
importFrom(gridExtra,tableGrob)
importFrom(gridExtra,ttheme_minimal)
importFrom(gtable,gtable_add_cols)
importFrom(gtable,gtable_add_grob)
importFrom(gtable,gtable_add_padding)
importFrom(gtable,gtable_add_rows)
importFrom(stats,na.omit)
importFrom(utils,modifyList)
