#' @keywords internal
"_PACKAGE"

# The following block is used by usethis to automatically manage
# roxygen namespace tags. Modify with care!
## usethis namespace: start
#' @import grid
#' @importFrom gtable gtable_add_grob gtable_add_rows gtable_add_padding gtable_add_cols
#' @importFrom gridExtra tableGrob ttheme_minimal
## usethis namespace: end
NULL
